#ifdef BORNAGAIN_PYTHON

#include "PyCore/Embed/PyCore.h"
#include "ImportMultiLayer.h"
#include "PyCore/Embed/PythonInterpreter.h" // BornAgain::importScript

// SWIG runtime access for creating a MultiLayer instance from Python
#include "auto/Wrap/swig_runtime.h"


namespace PythonInterpreter {

PyObjectPtr createMultiLayerFromPython(void*& multilayer_ptr,
                                       const std::string& script,
                                       const std::string& functionName,
                                       const std::string& path)
{
    PyObjectPtr tmpModule{PythonInterpreter::BornAgain::importScript(script, path)};

    // locate the `get_simulation` function (it is an attribute of the module)
    PyObject* pAddFn = PyObject_GetAttrString(tmpModule.get(), functionName.c_str());
    if (!pAddFn) {
        std::string msg = "PythonInterpreter::BornAgain: "
            "Cannot locate the compiled function '" + functionName + "'";
        return {Status(Status::Type::Error, msg)};
    }

    // create a `MultiLayer` Python object via calling the function
    PyObject* instance = PyObject_CallFunctionObjArgs(pAddFn, NULL);

    // clean up
    Py_DecRef(pAddFn);

    if (!instance) {
        std::string msg = "PythonInterpreter::BornAgain: "
            "Cannot call the function '" + functionName + "'";
        return {Status(Status::Type::Error, PythonInterpreter::errorDescription(msg))};
    }

    // Construct a C++ object from the Python object
    /* NOTE:
       This conversion requires a SWIG-produced Python API.
     */

    void* argp1 = 0;
    swig_type_info* pTypeInfo = SWIG_TypeQuery("MultiLayer *");

    const int res = SWIG_ConvertPtr(instance, &argp1, pTypeInfo, 0);
    if (!SWIG_IsOK(res)) {
        Py_DecRef(instance);
        std::string msg = "PythonInterpreter::BornAgain: SWIG failed to extract a "
            "'MultiLayer' instance via calling the function '" + functionName + "'";
        return {Status(Status::Type::Error, PythonInterpreter::errorDescription(msg))};
    }

    multilayer_ptr = reinterpret_cast<void*>(argp1);
    return {instance};
}

} // namespace PythonInterpreter

#endif // BORNAGAIN_PYTHON
