//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      PyCore/Sample/ImportMultiLayer.h
//! @brief     Forward declaration for createMultiLayerFromPython.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_PYCORE_SAMPLE_IMPORTMULTILAYER_H
#define BORNAGAIN_PYCORE_SAMPLE_IMPORTMULTILAYER_H

#ifndef SWIG

#include "PyCore/Embed/PyObjectPtr.h"
#include <string>

class MultiLayer;

namespace PythonInterpreter {

//! Creates a multi layer by running python code in embedded interpreter, given
//! The Python-script filename and the name of the function which produces a `MultiLayer`.
//! A path can be given to import `BornAgain` library (if empty, relies on `PYTHONPATH`).
//! Finally, if successful, `multilayer_ptr` will be a void pointer to the resulting
// `MultiLayer` instance in C++ (memory should be managed by the caller).
//! The returned value is the PyObject corresponding the intermediate _Python_ instance of
//! of the `MultiLayer` class.
PyObjectPtr createMultiLayerFromPython(void*& multilayer_ptr,
                                       const std::string& script,
                                       const std::string& functionName,
                                       const std::string& path = "");

} // namespace PythonInterpreter

#endif // SWIG

#endif // BORNAGAIN_PYCORE_SAMPLE_IMPORTMULTILAYER_H
